@@ -10,7 +10,6 @@ import shlex
 import inspect, traceback, importlib
 from collections import defaultdict, OrderedDict
 from functools import reduce, wraps
-from copy import deepcopy
 import shelve
 
 from math import ceil
@@ -202,7 +201,6 @@ class GramExp(object):
         self.lod = self._tensors._lod
 
     def update_default_expe(self, expformat):
-
         if not hasattr(expformat, '_default_expe'):
             return
         else:
@@ -215,7 +213,7 @@ class GramExp(object):
         # Use default _spec if no spec given
         _names = self.get_list('_expe_name')
         if '_spec' in default_expe and '_default_expe' in _names and len(_names) == 1:
-            specs = default_expe['_spec']
+            specs = default_expe.pop('_spec')
             specs = [specs] if not isinstance(specs, list) else specs
             group = []
 
@@ -226,6 +224,7 @@ class GramExp(object):
             for spec in specs:
                 if isinstance(spec, str):
                     d, expdesign = Spec.load(spec, self._spec[spec])
+                    default_expe["_expe_name"] = spec
                     group.append((spec, d, expdesign))
                 else:
                     # Assume dict/expspace
@@ -236,7 +235,7 @@ class GramExp(object):
             # update the lookup script
             if 'script' in conf:
                 script = conf.pop('script')
-                _script, script_args = Script.get(script[0], script[1:])
+                _, _script, script_args = Script.get(script[0], script[1:])
                 if script_args:
                     conf['_do'] = script_args
                     #self._tensors.update_all(_do=script_args)
@@ -294,7 +293,7 @@ class GramExp(object):
 
     def io_format_check(self):
         if len(self) > 1 and '_write' in self._conf:
-            self.check_format()
+            self.validate_format()
 
         # Clean pymake extra args:
         extra_args = ['_ignore_format_unique', ('_net', False)]
@@ -309,7 +308,7 @@ class GramExp(object):
         for key in keys_to_remove:
             self._tensors.remove_all(key)
 
-    def check_format(self):
+    def validate_format(self):
         ''' Check if all expVector are distinguishable in _format.
 
             @debug: not valid check accros tensors !!!
@@ -336,7 +335,7 @@ class GramExp(object):
                 print(' ' + '  '.join(hidden_key))
                 print('Possible conflicts in experience results outputs.')
                 print('Please correct {_format} key to fit the experience settings.')
-                print('To force the runs, use:  --ignore-format-unique')
+                print('To force the runs, use: --ignore-format-unique')
                 print('Exiting...')
                 exit(2)
 
@@ -554,7 +553,7 @@ class GramExp(object):
             # Assume pickle driver for models.
             filen = filen + '.' + 'pk'
 
-        if status is 'f' and is_empty_file(filen):
+        if status == 'f' and is_empty_file(filen):
             return None
         else:
             return filen
@@ -621,6 +620,35 @@ class GramExp(object):
 
         return fmt_expe
 
+    def _get_script(self):
+        if 'script' in self._conf:
+            script = self._conf.pop('script')
+            self._tensors.remove_all('script')
+            self._tensors.remove_all('_do')
+        else:
+            self.log.error('==> Error : You need to specify a script. (--script)')
+            exit(10)
+
+        try:
+            module, _script, script_args = Script.get(script[0], script[1:])
+        except (ValueError, TypeError) as e:
+            self.log.warning('Script not found, re-building Scripts indexes...')
+            self.update_index('script')
+            try:
+                res = Script.get(script[0], script[1:])
+                if not res:
+                    self.log.error('Unknown script: %s' % (script[0]))
+                    exit(404)
+                else:
+                    module, _script, script_args = res
+            except:
+                raise
+        except IndexError as e:
+            self.log.error('Script arguments error : %s -- %s' % (e, script))
+            exit(2)
+
+        return module, _script, script_args
+
     @classmethod
     def get_parser(cls, description=None, usage=None):
         import pymake.core.gram as _gram
@@ -645,7 +673,7 @@ class GramExp(object):
                 #@improve priority should be on : local project !
                 cls.log.warning("Project name `%s' seems to already exists in your PYTHONPATH.\n"
                                 "Project's name should not conflict with existing ones.\n"
-                                "Please change the name of your project's repo." % prjt)
+                                "Please verify or change the names of your project's repo and the gramarg.py file." % prjt)
                 exit(38)
 
         grammar = []
@@ -738,20 +766,21 @@ class GramExp(object):
         init                                                             init a pmk pymake repo.
         update                                                           update the pymake index.
         hist [-n n_lines]                                                show command history.
-        -l [spec(default)|model|script|topo]                             list available component.
+        -l [spec|model|script|topo]                             list available component.
         show SPEC                                 pmk SPEC               show one spec details (default if no arguments).
         run SPEC [--script [fun] [*args]]         pmk -x FUNC            execute tasks (default if -x is given).
         runpara SPEC [--script [fun] [*args]]     pmk -x FUNC --cores N  parallelize tasks (implicit if --cores is given).
         diff SPEC1 SPEC2                                                 show diff between two spec.
         cmd SPEC                                                         try to generate the command-line for each expe.
         path SPEC Filetype(pk|json|inf) [status]                         show output_path of each expe.
+        pmk doc|help -x SCRIPT                                           show the doc of the given script
         '''
 
         s, parser, expdesign_lkp = GramExp.parseargsexpe(usage)
         request.update(s)
 
         ontology = dict(
-            _do=['cmd', 'show', 'path', 'run', 'update', 'init', 'runpara', 'hist', 'diff'],
+            _do=['cmd', 'show', 'path', 'run', 'update', 'init', 'runpara', 'hist', 'diff', 'doc', 'help'],
             _spec=list(cls._spec),
             _ext=['json', 'pk', 'inf']
         )
@@ -926,8 +955,8 @@ class GramExp(object):
     #            break
     #    return
 
-    def exptable(self):
-        return self._tensors.table()
+    def modeltable(self, _type='short'):
+        return Model.table(_type)
 
     def spectable(self):
         return Spec.table()
@@ -935,20 +964,22 @@ class GramExp(object):
     def scripttable(self):
         return Script.table()
 
-    def modeltable(self, _type='short'):
-        return Model.table(_type)
-
     def spectable_topo(self):
         return Spec.table_topos(self._spec)
 
     def alltable_topo(self):
-        from pymake.core.types import _table_
-        specs = self._spec
-        scripts = Script.get_all()
-        models = Model.get_all()
-        table = [models, specs, scripts]
-        headers = ['Models', 'Specs', 'Actions']
-        return _table_(table, headers)
+        sep = '\n'+'='*20+'\n'
+        #from pymake.core.types import _table_
+        #specs = self._spec
+        #scripts = Script.get_all()
+        #models = Model.get_all()
+        #table = [models, specs, scripts[1:]]
+        #headers = ['Models', 'Specs', 'Actions']
+        #return _table_(table, headers)
+        return sep.join(map(lambda x: str(x.table()), [Model, Spec, Script]))
+
+    def exptable(self):
+        return self._tensors.table()
 
     def help_short(self):
         shelp = self.argparser.format_usage() + self.argparser.epilog
@@ -965,9 +996,11 @@ class GramExp(object):
 
     def simulate(self, halt=True, file=sys.stdout):
 
-        print('PYMAKE Exp: %d' % (len(self)), file=file)
-        print('-'*30, file=file)
-        print(self.exptable(), file=file)
+        if not self._conf.get("_expe_silent"):
+            print('PYMAKE Exp: %d' % (len(self)), file=file)
+            print('-'*30, file=file)
+            print(self.exptable(), file=file)
+
         if halt:
             exit()
         else:
@@ -1094,33 +1127,7 @@ class GramExp(object):
 
     def execute(self):
         ''' Execute Exp Sequentially. '''
-
-        if 'script' in self._conf:
-            script = self._conf.pop('script')
-            self._tensors.remove_all('script')
-            self._tensors.remove_all('_do')
-        else:
-            self.log.error('==> Error : You need to specify a script. (--script)')
-            exit(10)
-
-        try:
-            _script, script_args = Script.get(script[0], script[1:])
-        except (ValueError, TypeError) as e:
-            self.log.warning('Script not found, re-building Scripts indexes...')
-            self.update_index('script')
-            try:
-                res = Script.get(script[0], script[1:])
-                if not res:
-                    self.log.error('Unknown script: %s' % (script[0]))
-                    exit(404)
-                else:
-                    _script, script_args = res
-            except:
-                raise
-        except IndexError as e:
-            self.log.error('Script arguments error : %s -- %s' % (e, script))
-            exit(2)
-
+        _, _script, script_args = self._get_script()
         if script_args:
             self._tensors.update_all(_do=script_args)
         self.pymake(sandbox=_script)
@@ -1457,6 +1464,20 @@ class GramExp(object):
 
         exit()
 
+    def show_doc(self):
+        self._conf["_expe_silent"] = True
+        module, script, _ = self._get_script()
+        docs = []
+        if module.__doc__:
+            docs.append(module.__doc__)
+        if script.__doc__:
+            docs.append(script.__doc__)
+
+        if docs:
+            return docs
+
+        return "No doc here. Condider adding a __doc__  in your file..."
+
     @classmethod
     def update_index(cls, *index_name):
         from pymake.index.indexmanager import IndexManager as IX
@@ -1525,6 +1546,7 @@ class GramExp(object):
         # get Corpus
         corpus = None
 
+        spec_cmds = "|".join(["pmk", "run", "runpara", "show", "path", "diff", "doc"])
         all_scripts = ' '.join(all_scripts)
         sur_scripts = ' '.join(sur_scripts)
         dict_scripts = '(' + ' '.join(dict_scripts) + ')'
@@ -1561,6 +1583,7 @@ class GramExp(object):
         with open(completion_fn, 'w') as _f:
             template = PmkTemplate(template)
             template = template.substitute(projectname=prjt, version=__version__,
+                                           spec_cmds=spec_cmds,
                                            specs=specs,
                                            all_scripts=all_scripts,
                                            sur_scripts=sur_scripts,
